#!/usr/bin/env python3
# Replaces a map inside a DDNet demo file.
# Command line parameters: INPUT-DEMO MAP OUTPUT-DEMO
#
# Author: Patiga
# License: CC-BY

import sys
import hashlib

with open(sys.argv[1], "rb") as f:
    data = bytearray(f.read())
with open(sys.argv[2], "rb") as f:
    twmap = f.read()

MAP_SIZE_POS = 136
CRC_POS = MAP_SIZE_POS + 4
DEMO_KIND_POS = CRC_POS + 4
UUID = bytes([
    0x6b, 0xe6, 0xda, 0x4a, 0xce, 0xbd, 0x38, 0x0c,
	0x9b, 0x5b, 0x12, 0x89, 0xc8, 0x42, 0xd7, 0x80,
])
UUID_POS = DEMO_KIND_POS + 292
SHA_POS = UUID_POS + 16
MAP_POS = SHA_POS + 32

assert(data[:7] == b"TWDEMO\0")
version = data[7]
assert(version == 6)
data[CRC_POS:CRC_POS+4] = [0] * 4
assert(data[DEMO_KIND_POS:DEMO_KIND_POS+8] == b"client\0\0")
map_size = int.from_bytes(data[MAP_SIZE_POS:MAP_SIZE_POS+4], "big")
data[MAP_SIZE_POS:MAP_SIZE_POS+4] = len(twmap).to_bytes(4, "big")
assert(data[UUID_POS:UUID_POS+16] == UUID)

sha = hashlib.sha256()
sha.update(twmap)
sha = sha.digest()
data[SHA_POS:SHA_POS+32] = sha
data = data[:MAP_POS] + twmap + data[MAP_POS + map_size:]

with open(sys.argv[3], "wb") as f:
    f.write(data)
